from django.db import migrations, models
import django.db.models.deletion
import wagtail.contrib.routable_page.models
import wagtail.core.blocks
import wagtail.core.fields
import wagtail.images.blocks
from wagtail.images import get_image_model_string


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('wagtailcore', '0040_page_draft_title'),
        migrations.swappable_dependency(get_image_model_string())
    ]

    operations = [
        migrations.CreateModel(
            name='Event',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('start_date', models.DateTimeField()),
                ('end_date', models.DateTimeField(blank=True, null=True)),
                ('description', models.TextField(blank=True, help_text='Briefly describe your event series', max_length=400)),
                ('body', wagtail.core.fields.StreamField([('heading', wagtail.core.blocks.CharBlock(help_text="Add optional subheadings between paragraphs, if you're describing the event series in more detail", label='Optional extra description subheading', max_length=120, required=False)), ('paragraph', wagtail.core.blocks.TextBlock(help_text='Tell people about the event series', label='Optional extra description text', required=False)), ('image', wagtail.images.blocks.ImageChooserBlock(help_text='Add images to describe the event series.', label='Optional extra event series image(s)', required=False)), ('quote', wagtail.core.blocks.BlockQuoteBlock(help_text='Add an inspirational quote!', required=False))], blank=True, help_text='Add a description of this event or event series.')),
                ('image', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=get_image_model_string())),
            ],
            options={
                'ordering': ['start_date'],
                'abstract': False,
            },
            bases=(wagtail.contrib.routable_page.models.RoutablePageMixin, 'wagtailcore.page'),
        ),
        migrations.CreateModel(
            name='EventIndex',
            fields=[
                ('page_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='wagtailcore.Page')),
                ('paginate_by', models.PositiveIntegerField(blank=True, null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=(wagtail.contrib.routable_page.models.RoutablePageMixin, 'wagtailcore.page'),
        ),
    ]



import datetime
import re
from django.core.exceptions import ValidationError
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.db import models
from django.utils import timezone
from wagtail.admin.edit_handlers import FieldPanel, MultiFieldPanel
from wagtail.core.models import Page

from wagtail_events import date_filters
from wagtail_events.managers import DatedEventManager
from wagtail_events import utils
from wagtail_events.utils import _DATE_FORMAT_RE


class AbstractPaginatedIndex(Page):
    """ """
    paginate_by = models.PositiveIntegerField(blank=True, null=True)
    content_panels = Page.content_panels + [FieldPanel('paginate_by')]
    paginator_class = Paginator

    class Meta(object):
        """Django model meta options."""
        abstract = True

    def _get_children(self, request, *args, **kwargs):
        """
        Helper method for getting child nodes to display in the listing.

        :param request: django request
        :return: Queryset of child model instances
        """
        model_class = self.__class__.allowed_subpage_models()[0]
        children = model_class.objects.child_of(self)
        if not request.is_preview:
            children = children.filter(live=True)

        # Children should only be displayed if show_in_menus is true.
        children = children.filter(show_in_menus=True)

        return children

    def get_paginator_class(self):
        """
        Returns the class to use for pagination

        :return: Paginator class
        """
        return self.paginator_class

    def get_paginator_kwargs(self):
        """
        Method for generating a dict of keyword args that will be
        passed to the paginator constructor

        :param request: HttpRequest instance
        :return: Dict of keyword arguments to pass to the paginator class constructor
        """
        return {}

    def get_paginator(self, *args, **kwargs):
        """
        Returns a paginator instance

        :return: Paginator class
        """
        paginator_class = self.get_paginator_class()
        return paginator_class(*args, **kwargs)

    def paginate_queryset(self, queryset, page):
        """
        Helper method for paginating the queryset provided.

        :param queryset: Queryset of model instances to paginate
        :param page: Raw page number taken from the request dict
        :return: Queryset of child model instances
        """
        paginator = self.get_paginator(
            queryset,
            self.paginate_by,
            **self.get_paginator_kwargs()
        )
        try:
            queryset = paginator.page(page)
        except PageNotAnInteger:
            queryset = paginator.page(1)
        except EmptyPage:
            queryset = paginator.page(paginator.num_pages)
        return queryset, paginator

    def get_context(self, request, *args, **kwargs):
        """
        Adds child pages to the context and paginates them.

        :param request: HttpRequest instance
        :param args: default positional args
        :param kwargs: default keyword args
        :return: Context data to use when rendering the template
        """
        context = super(AbstractPaginatedIndex, self).get_context(
            request,
            *args,
            **kwargs
        )
        queryset = self._get_children(request, *args, **kwargs)
        is_paginated = False
        paginator = None

        # Paginate the child nodes if paginate_by has been specified
        if self.paginate_by:
            is_paginated = True
            page_num = request.GET.get('page', 1) or 1
            queryset['items'], paginator = self.paginate_queryset(
                queryset['items'],
                page_num
            )

        context.update(
            children=queryset,
            paginator=paginator,
            is_paginated=is_paginated
        )
        return context


class AbstractEventIndex(AbstractPaginatedIndex):

    class Meta(object):
        abstract = True

    def get_dateformat(self):
        """
        Returns the date format regex
        """
        return _DATE_FORMAT_RE

    def _get_children(self, request, *args, **kwargs):
        """
        :param request: django request
        :return: Queryset of child model instances, filtered by a scope and start date in the request params
        """
        qs = super(AbstractEventIndex, self)._get_children(request).specific()

        time_periods = {
            'year': date_filters.get_year_range,
            'week': date_filters.get_week_range,
            'month': date_filters.get_month_range,
            'day': date_filters.get_day_range,
        }
        period = request.GET.get('scope', None)

        if period:

            # Get the start date from the request (or use default now)
            start_date = request.GET.get('start_date', '')
            if re.match(self.get_dateformat(), start_date):
                date_params = [int(i) for i in start_date.split('.')]
                start_date = utils.date_to_datetime(datetime.date(*date_params))
            else:
                start_date = timezone.now().replace(
                    hour=0,
                    minute=0,
                    second=0,
                    microsecond=0,
                )

            # Clean the start and end dates to conform to the requested period
            start_date, end_date = time_periods[period.lower()](start_date)

            # Filter the original queryset to the date range specified
            qs.filter(start_date__gte=start_date).filter(end_date__lte=end_date)

        return qs


class AbstractEvent(Page):

    start_date = models.DateTimeField()
    end_date = models.DateTimeField(blank=True, null=True)
    objects = DatedEventManager()

    content_panels = [
        MultiFieldPanel(
            [
                FieldPanel('start_date'),
                FieldPanel('end_date'),
            ],
            heading="Event Start / End Dates"
        ),
    ]

    parent_page_types = ['wagtail_events.EventIndex']
    subpage_types = []

    class Meta(object):
        abstract = True
        ordering = ['start_date']

    def clean(self):
        """Clean the model fields, if end_date is before start_date raise a ValidationError."""
        super(AbstractEvent, self).clean()
        if self.end_date:
            if self.end_date < self.start_date:
                raise ValidationError({'end_date': 'The end date cannot be before the start date.'})



# Generated by Django 2.1.4 on 2019-03-28 13:59

from django.db import migrations, models
import wagtail.core.blocks
import wagtail.core.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('wagtail_events', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='event',
            name='body',
            field=wagtail.core.fields.StreamField([('heading', wagtail.core.blocks.CharBlock(help_text="Add optional subheadings between paragraphs, if you're describing the event series in more detail", label='Subheading', max_length=120, required=False)), ('paragraph', wagtail.core.blocks.TextBlock(help_text='Tell people about the event series', label='Text', required=False)), ('image', wagtail.images.blocks.ImageChooserBlock(help_text='Add images to describe the event series.', label='Image', required=False)), ('quote', wagtail.core.blocks.BlockQuoteBlock(help_text='Add an inspirational quote!', label='Quote', required=False))], blank=True, help_text='Optional: Add further description of this event.'),
        ),
        migrations.AlterField(
            model_name='event',
            name='description',
            field=models.TextField(blank=True, help_text='Briefly describe your event', max_length=400),
        ),
    ]



from datetime import datetime
from isoweek import Week
from wagtail_events import utils


def get_year_range(start_date):
    """
    Get the start and end datetimes for the year

    :param start_date: period start_date
    :type start_date: datetime.datetime()
    :return: tuple start_datetime, end_datetime
    """
    start_date = datetime(start_date.year, 1, 1)
    end_date = utils.date_to_datetime(utils.add_months(start_date, 12), 'max')
    return start_date, end_date


def get_month_range(start_date):
    """
    Get the start and end datetimes for the month

    :param start_date: period start_date
    :type start_date: datetime.datetime()
    :return: tuple start_datetime, end_datetime
    """
    start_date = datetime(start_date.year, start_date.month, 1)
    end_date = utils.date_to_datetime(
        utils.add_months(start_date.date(), 1),
        'max'
    )
    return start_date, end_date


def get_week_range(start_date):
    """
    Get the start and end datetimes for the week

    :param start_date: period start_date
    :type start_date: datetime.datetime()
    :return: tuple start_datetime, end_datetime
    """
    period = Week(start_date.year, start_date.date().isocalendar()[1])
    start_date = utils.date_to_datetime(period.monday())
    end_date = utils.date_to_datetime(period.sunday(), 'max')
    return start_date, end_date


def get_day_range(start_date):
    """
    Get the start and end datetimes for the day

    :param start_date: period start_date
    :type start_date: datetime.datetime()
    :return: tuple start_datetime, end_datetime
    """
    start_date = utils.date_to_datetime(start_date.date(), 'min')
    end_date = utils.date_to_datetime(start_date.date(), 'max')
    return start_date, end_date



# -*- coding:utf8 -*-

from __future__ import unicode_literals

try:
    from urllib.parse import urlencode
except ImportError:
    from urllib import urlencode

from django import template


register = template.Library()


@register.simple_tag(takes_context=True)
def querystring(context, *args, **kwargs):
    """
    Display all GET values (except page) encoded as url params

    :param context: template context
    :return: string|encoded params as urlstring
    """
    try:
        params = context['request'].GET.dict()
    except (KeyError, AttributeError):
        params = {}
    else:
        for value in args:
            params.pop(value, None)
        for key, value in kwargs.items():
            params[key] = value
    return urlencode(params)


def _patch(context, key, data):
    """
    Patch the GET value

    :param context: template context dict
    :param key: item name
    :param data: item value
    :return: patched url params
    """
    getvars = dict(context['request'].GET)
    getvars[key] = [data]
    return '?{0}'.format(urlencode(getvars, doseq=True))


@register.simple_tag(takes_context=True)
def patch_scope(context, scope):
    """
    Prepare scope for agenda

    :param context:
    :param scope:
    :return:
    """
    return _patch(context, 'scope', scope)


@register.simple_tag(takes_context=True)
def patch_start_date(context, date):
    """
    Prepare `start_date` url for agenda

    :param context: template context dict
    :param date: start_date
    :return:
    """
    return _patch(context, 'start_date', date.strftime('%Y.%m.%d'))



from django.db import models
from wagtail.admin.edit_handlers import FieldPanel, StreamFieldPanel, MultiFieldPanel
from wagtail.core.blocks import CharBlock, TextBlock, BlockQuoteBlock
from wagtail.core.fields import StreamField
from wagtail.core.models import Page
from wagtail.images.blocks import ImageChooserBlock
from wagtail.images.edit_handlers import ImageChooserPanel


from wagtail_events.abstract_models import AbstractEvent, AbstractEventIndex


def get_image_model_path():
    from django.conf import settings
    return getattr(settings, 'WAGTAILIMAGES_IMAGE_MODEL', 'wagtailimages.Image')


class Event(AbstractEvent):

    image = models.ForeignKey(get_image_model_path(), null=True, blank=True, on_delete=models.SET_NULL, related_name='+')
    description = models.TextField(max_length=400, help_text='Briefly describe your event', null=False, blank=True)
    body = StreamField([
        ('heading', CharBlock(required=False, label='Subheading', max_length=120, help_text="Add optional subheadings between paragraphs, if you're describing the event series in more detail")),
        ('paragraph', TextBlock(required=False, label='Text', help_text='Tell people about the event series')),
        ('image', ImageChooserBlock(required=False, label='Image', help_text='Add images to describe the event series.')),
        ('quote', BlockQuoteBlock(required=False, label='Quote', help_text='Add an inspirational quote!')),
    ], blank=True, help_text='Optional: Add further description of this event.')

    content_panels = Page.content_panels + [
        MultiFieldPanel(
            [
                FieldPanel('description'),
                ImageChooserPanel('image'),
            ],
            heading="Event Description"
        )] + AbstractEvent.content_panels + [
        StreamFieldPanel('body', heading='Extra Information')
    ]

    parent_page_types = ['wagtail_events.EventIndex']
    subpage_types = []


class EventIndex(AbstractEventIndex):
    parent_page_types = []
    subpage_types = ['wagtail_events.Event']
